
InitEnemyManager
; parse every type of enemy, init its life, state, char index and position
; update total and active enemies

; parse enemy type 1 (only one for now)
	ldx #1		; enemy type index
	lda TabEnemyDefBaseChar,x
	sta m_searchEnemyChar
	lda TabEnemyDefLife,x
	sta m_defEnemyLife
	
	ldx m_levelCharSizeX
	dex			; we search in a shorter line because this enemy has a width of 2 chars
	stx m_numberOfValidRows
	
	ldx #0		; line index

IEM_loop1
	ldy #0		; column index
	
	lda TabLevelLineAddressLSB,x
	sta m_ptrLevelLine
	lda TabLevelLineAddressMSB,x
	sta m_ptrLevelLine+1

IEM_loop2
	lda (m_ptrLevelLine),y
	cmp m_searchEnemyChar
	bne IEM_no_enemy

	stx m_tempByte
	ldx m_totalEnemies

; init info for this enemy
	lda #1
	sta TabEnemyType,x
	lda m_defEnemyLife
	sta TabEnemyLife,x
	lda #1
	sta TabEnemyState,x
	lda #ENEMY_INIT_TIMER					; dont allow enemy actions for the first 4 seconds
	sta TabEnemyStateTimer,x
	lda m_searchEnemyChar
	sta TabEnemyBaseCharIndex,x
	tya
	sta TabEnemyBaseCharX,x
	lda m_tempByte
	sta TabEnemyBaseCharY,x

; init level position
; TabEnemyLevelTopY_H2 starts as 0, now multiply by 8 to get the line
	asl
	rol TabEnemyLevelTopY_H2,x
	asl
	rol TabEnemyLevelTopY_H2,x
	asl
	rol TabEnemyLevelTopY_H2,x
	sta TabEnemyLevelTopY_H1,x
	
; TabEnemyLevelLeftX_H2 starts as 0, now multiply by 4 to get the horizontal pixel
	tya
	asl
	rol TabEnemyLevelLeftX_H2,x
	asl
	rol TabEnemyLevelLeftX_H2,x
	sta TabEnemyLevelLeftX_H1,x

; init shot creation position (offset x: 3, offset y: 1)
	lda TabEnemyLevelLeftX_H1,x
	clc
	adc #3
	sta TabEnemyShotLevelLeftX_H1,x
	lda TabEnemyLevelLeftX_H2,x
	adc #0
	sta TabEnemyShotLevelLeftX_H2,x
	
	lda TabEnemyLevelTopY_H1,x
	clc
	adc #1
	sta TabEnemyShotLevelTopY_H1,x
	lda TabEnemyLevelTopY_H2,x
	adc #0
	sta TabEnemyShotLevelTopY_H2,x

; update number of enemies	
	inc m_totalEnemies
	inc m_activeEnemies

	lda m_totalEnemies
	cmp #MAX_ENEMIES_PER_LEVEL
	beq IEM_exit
	
	ldx m_tempByte

IEM_no_enemy
	iny
	cpy m_numberOfValidRows
	beq IEM_check_loop_Y
	jmp IEM_loop2

IEM_check_loop_Y
	inx
	cpx m_levelCharSizeY
	beq IEM_exit
	jmp IEM_loop1

IEM_exit
	rts
	

UpdateEnemyManager

; update one enemy per frame, rotate through all active enemies
	lda m_activeEnemies
	bne UEM_active_enemies
	jmp UEM_exit

UEM_active_enemies
; update state timer for all enemies (if we don't want to do this, we should use a general time stamp system)
	ldx #0
UEM_loop1
	lda TabEnemyStateTimer,x
	beq UEM_next_timer
	dec TabEnemyStateTimer,x
UEM_next_timer
	inx
	cpx m_totalEnemies
	bne UEM_loop1

; iterate until we find the next active enemy
	ldx m_lastEnemyIndexUpdated
	
UEM_loop2
	inx
	cpx m_totalEnemies
	bne UEM_no_loop_back
	ldx #0
UEM_no_loop_back
	lda TabEnemyState,x		; state 0 implies a disabled enemy
	beq UEM_loop2
	stx m_lastEnemyIndexUpdated

; should use the enemy type to jump to the correct update for that type

; check state timer, update state if is 0
	lda TabEnemyStateTimer,x
	bne UEM_exit

; timer is 0, check if the player is near enough to shoot, in what direction, and if there are missiles available
; (maybe check against the maximum number of enemy missiles possible)
UEM_check_shoot
	lda m_enemyMissilesActive
	cmp #MAX_ENEMY_MISSILES
	beq UEM_exit
	
; check delta Y between the player and the enemy
	lda m_playerLevelTopY_H1
	sec
	sbc TabEnemyLevelTopY_H1,x
	sta m_pageZeroTempPointer
	lda m_playerLevelTopY_H2
	sbc TabEnemyLevelTopY_H2,x
	sta m_pageZeroTempPointer+1

	lda m_pageZeroTempPointer
	clc
	adc #ENEMY_SHOOT_DELTA_Y
	sta m_pageZeroTempPointer
	lda m_pageZeroTempPointer+1
	adc #0
	sta m_pageZeroTempPointer+1
	bne UEM_exit	; abort if the high byte is already greater than 0

; this works for a delta Y lower than 128 (lines)
	lda m_pageZeroTempPointer
	cmp #[ENEMY_SHOOT_DELTA_Y*2]
	bcs UEM_exit
	
UEM_shot
	ldy TabEnemyType,x
	lda TabEnemyDefShootCooldown,y
	sta TabEnemyStateTimer,x					; set shoot cooldown

; update m_enemyShootDirection (1 - right, 2 - left)

; player width = 4 pixels, enemy1 width = 8 pixels
; compare to see if: px + (pw/2) == ex + (ew/2) (the center of the player and the enemy)
; if player info is equal or greater, then player is to the right of the enemy
	lda m_playerLevelLeftX_H1
	sec
	sbc #4
	sta m_pageZeroTempValue
	lda m_playerLevelLeftX_H2
	sbc #0

	cmp TabEnemyLevelLeftX_H2,x
	bcc UEM_shoot_left
	bne UEM_shoot_right
	
	lda m_pageZeroTempValue
	cmp TabEnemyLevelLeftX_H1,x
	bcc UEM_shoot_left
	
UEM_shoot_right
	lda #1
	jmp UEM_just_shoot
UEM_shoot_left
	lda #2
UEM_just_shoot
	sta m_enemyShootDirection

; update shot creation pos (assuming left and right pos are the same in this case)
	lda TabEnemyShotLevelLeftX_H1,x
	sta m_enemyShotLevelLeftX_H1
	lda TabEnemyShotLevelLeftX_H2,x
	sta m_enemyShotLevelLeftX_H2
	
	lda TabEnemyShotLevelTopY_H1,x
	sta m_enemyShotLevelTopY_H1
	lda TabEnemyShotLevelTopY_H2,x
	sta m_enemyShotLevelTopY_H2

	jsr CheckCreateEnemyMissile
	
	inc m_enemyMissilesActive
	
UEM_exit
/*
	lda m_lastEnemyIndexUpdated
	ldy #23
	jsr DisplayDebugInfoHexFF	
*/
	rts

	
;================================================================================

HitEnemy

; should not use x register
	stx m_hitEnemySaveX

; pass parameters in m_enemyCharPosX and m_enemyCharPosY
; get address of that char (clear it and the next one) --> comes in m_lineAddressLSB, m_lineAddressMSB
; get pixel xpos for pm hpos, get pixel ypos for explosion start line

; search for enemy index at the left char position
	ldx #0

HE_loop	
	lda TabEnemyBaseCharX,x
	cmp m_enemyCharPosX
	bne HE_next_enemy
	lda TabEnemyBaseCharY,x
	cmp m_enemyCharPosY
	bne HE_next_enemy

HE_found_pos
	lda TabEnemyState,x
	beq HE_exit				; already disabled
	
	lda TabEnemyLife,x
	beq HE_exit				; already destroyed

	dec TabEnemyLife,x
	beq HE_destroy_enemy	; life reached 0
	
	jmp HE_exit				; still alive (no feedback for now?)

HE_next_enemy
	inx
	cpx m_totalEnemies
	bne HE_loop
	jmp HE_exit
	
HE_destroy_enemy
	lda #0
	sta TabEnemyState,x
	
	dec m_activeEnemies
	
	lda m_lineAddressLSB
	sta m_ptrLevelLine
	lda m_lineAddressMSB
	sta m_ptrLevelLine+1
	
	ldy m_enemyCharPosX
	lda #0
	sta (m_ptrLevelLine),y
	iny
	sta (m_ptrLevelLine),y

; init explosion info (clear previous explosion if there was one active)
	lda m_numEnemyExplosions
	beq HE_no_explosions
	
	jsr EraseExplosionFrame

HE_no_explosions
	lda TabEnemyLevelLeftX_H2,x
	sta m_enemyExplosionLevelLeftX_H2
	lda TabEnemyLevelLeftX_H1,x
	sta m_enemyExplosionLevelLeftX_H1
	
	lda TabEnemyLevelTopY_H2,x
	sta m_enemyExplosionLevelTopY_H2
	lda TabEnemyLevelTopY_H1,x
	sta m_enemyExplosionLevelTopY_H1
	lda #0
	sta m_enemyExplosionLevelTopY_L1

	lda #1
	sta m_enemyExplosionStep
	lda #0
	sta m_enemyExplosionStepIndex

	lda #1
	sta m_numEnemyExplosions		; 1 max for now

HE_exit
	ldx m_hitEnemySaveX
	
	rts

;------------------------------------------------------------
; also clear enemy explosion if the player respawns
; also clear enemy missiles and m_enemyMissilesActive

EraseExplosionFrame
	ldx #0
	ldy m_enemyExplosionScreenTopY_H1

EEF_loop
	lda #0
	sta p1_area_1,y

	tya
	clc
	adc m_enemyExplosionStep
	;bcs EEF_exit				; comment this to allow "wrap around"
	tay

	inx
	cpx TabEnemyShapeSizeY		; should use and index here, if there is more than one enemy explosion
	bne EEF_loop

EEF_exit
	rts

;------------------------------------------------------------
DrawExplosionFrame
	ldx #0
	ldy m_enemyExplosionScreenTopY_H1

DEF_loop
	lda TabEnemyShape_01,x
	sta p1_area_1,y

	tya
	clc
	adc m_enemyExplosionStep
	;bcs DEF_exit				; comment this to allow "wrap around"
	tay

	inx
	cpx TabEnemyShapeSizeY		; should use and index here, if there is more than one enemy explosion
	bne DEF_loop

DEF_exit
	rts

;------------------------------------------------------------
DrawEnemyExplosion
	lda m_numEnemyExplosions
	bne DEE_erase
	jmp DEE_exit
	
; erase previous enemy explosion frame
DEE_erase
	jsr EraseExplosionFrame

DEE_update_frame
	inc m_enemyExplosionStepIndex
	ldx m_enemyExplosionStepIndex
	cpx #MAX_EXPLOSION_STEPS
	bcc DEE_do_update
	
; reached end of the explosion
	lda #0
	sta m_numEnemyExplosions

	jmp DEE_exit

DEE_do_update
	lda TabEnemyExplosionColor,x
	sta COLPM1

; update step and explosion starting line
; if step +1 --> line - 3.5
; if step +2 --> line - 7.0
	lda m_enemyExplosionLevelTopY_L1
	sec
	sbc #128
	sta m_enemyExplosionLevelTopY_L1

	lda m_enemyExplosionLevelTopY_H1
	sbc #3
	sta m_enemyExplosionLevelTopY_H1

	lda m_enemyExplosionLevelTopY_H2
	sbc #0
	sta m_enemyExplosionLevelTopY_H2

	inc m_enemyExplosionStep	; step +1 (delta between every explosion line, in this frame)
;	inc m_enemyExplosionStep	; step +2

; get new screen pos for the explosion
; this must use the "_H2" positions for when the delta between the explosion and the camera is too big (64 chars in X, or 32 chars in Y)

	lda m_enemyExplosionLevelLeftX_H1
	sec
	sbc m_levelWindowX_H1
	sta m_screenDelta_H1

	lda m_enemyExplosionLevelLeftX_H2
	sbc m_levelWindowX_H2
	bcc DEE_set_outside			; this is not totally correct, because it doesn't use the width of the missile
	sta m_screenDelta_H2		; (but if we do it correctly we should clip the missile graphics)

	bne DEE_set_outside			; could remove the bcc and only use this..
	lda m_screenDelta_H1
	cmp #[160]					; 40 chars x 4 pixels
	bcs DEE_set_outside
	
	adc #PM_BASE_SCREEN_X
	sta m_enemyExplosionScreenLeftX_H1
	

	lda m_enemyExplosionLevelTopY_H1
	sec
	sbc m_levelWindowY_H1
/*	sta m_screenDelta_H1
	
	lda m_enemyExplosionLevelTopY_H2
	sbc m_levelWindowY_H2
	bcc DEE_set_outside			; this is not totally correct, because it doesn't use the height of the missile
	sta m_screenDelta_H2		; (but if we do it correctly we should clip the missile graphics)

	bne DEE_set_outside			; could remove the bcc and only use this..
	lda m_screenDelta_H1
	cmp #[193]					; 24 chars x 8 scan lines, plus one extra scan line
	bcs DEE_set_outside
*/
	adc #PM_BASE_SCREEN_Y
	sta m_enemyExplosionScreenTopY_H1

	jmp DEE_update_hpos


; if outside the visible range in X or Y, just set it out of the screen (for a p/m the screen coordinate 0 is outside the visible range)
DEE_set_outside
	lda #0
	sta m_enemyExplosionScreenLeftX_H1
	sta m_enemyExplosionScreenTopY_H1


DEE_update_hpos
	lda m_enemyExplosionScreenLeftX_H1
	sta HPOSP1

; draw new enemy explosion frame
DEE_draw
	jsr DrawExplosionFrame


DEE_exit
	rts

;------------------------------------------------------------
m_enemyExplosionLevelLeftX_H2	.byte 0
m_enemyExplosionLevelLeftX_H1	.byte 0

m_enemyExplosionLevelTopY_H2	.byte 0
m_enemyExplosionLevelTopY_H1	.byte 0
m_enemyExplosionLevelTopY_L1	.byte 0

m_enemyExplosionScreenLeftX_H1	.byte 0
m_enemyExplosionScreenTopY_H1	.byte 0

m_enemyExplosionStep			.byte 0

m_enemyExplosionStepIndex		.byte 0
MAX_EXPLOSION_STEPS = 22

m_enemyCharPosX					.byte 0
m_enemyCharPosY					.byte 0

m_numEnemyExplosions			.byte 0


TabEnemyShapeSizeY
	.byte 8

TabEnemyShape_01
	.byte %00111100
	.byte %11100111
	.byte %00011000
	.byte %11100111
	.byte %00111100
	.byte %00000000
	.byte %00111100
	.byte %01111110

TabEnemyExplosionColor
.ifndef PAL_VERSION
	.by +32, 14, 14, 14, 12, 12, 12, 10, 10, 10, 8, 8, 8, 6, 6, 6, 4, 4, 4, 2, 2, 2, 0
.else
	.by +16, 14, 14, 14, 12, 12, 12, 10, 10, 10, 8, 8, 8, 6, 6, 6, 4, 4, 4, 2, 2, 2, 0
.endif

;================================================================================

MAX_ENEMIES_PER_LEVEL = 32
MAX_ENEMY_MISSILES = 2

.ifndef PAL_VERSION
ENEMY_EXPLOSION_COLOR = 32+10
.else
ENEMY_EXPLOSION_COLOR = 16+10
.endif

m_hitEnemySaveX .byte 0

m_totalEnemies .byte 0
m_activeEnemies .byte 0

m_searchEnemyChar .byte 0
m_defEnemyLife .byte 0
m_numberOfValidRows .byte 0

m_tempByte .byte 0

m_lastEnemyIndexUpdated .byte 0

m_enemyMissilesActive .byte 0

m_enemyShootDirection .byte 0

m_enemyShotLevelLeftX_H1 .byte 0
m_enemyShotLevelLeftX_H2 .byte 0
m_enemyShotLevelTopY_H1 .byte 0
m_enemyShotLevelTopY_H2 .byte 0


TabEnemyType
	:MAX_ENEMIES_PER_LEVEL .byte 0

TabEnemyLife		; this table could be per level, if we can go back to the same screen
	:MAX_ENEMIES_PER_LEVEL .byte 0

TabEnemyState
	:MAX_ENEMIES_PER_LEVEL .byte 0
TabEnemyStateTimer
	:MAX_ENEMIES_PER_LEVEL .byte 0

TabEnemyBaseCharIndex
	:MAX_ENEMIES_PER_LEVEL .byte 0
TabEnemyBaseCharX
	:MAX_ENEMIES_PER_LEVEL .byte 0
TabEnemyBaseCharY
	:MAX_ENEMIES_PER_LEVEL .byte 0

TabEnemyLevelLeftX_H1
	:MAX_ENEMIES_PER_LEVEL .byte 0
TabEnemyLevelLeftX_H2
	:MAX_ENEMIES_PER_LEVEL .byte 0
TabEnemyLevelTopY_H1
	:MAX_ENEMIES_PER_LEVEL .byte 0
TabEnemyLevelTopY_H2
	:MAX_ENEMIES_PER_LEVEL .byte 0

TabEnemyShotLevelLeftX_H1
	:MAX_ENEMIES_PER_LEVEL .byte 0
TabEnemyShotLevelLeftX_H2
	:MAX_ENEMIES_PER_LEVEL .byte 0
TabEnemyShotLevelTopY_H1
	:MAX_ENEMIES_PER_LEVEL .byte 0
TabEnemyShotLevelTopY_H2
	:MAX_ENEMIES_PER_LEVEL .byte 0


; enemy definition tables (index or type 0 is not used)
TabEnemyDefLife
	.byte 0, 1

TabEnemyDefBaseChar
	.byte 0, 192

TabEnemyDefShootCooldown
.ifndef PAL_VERSION
	.byte 0, 120
.else
	.byte 0, 100
.endif
