;================================================================================
; Pro-P1, Atari XL + mads [NRV 2010 - 2017]
;================================================================================

; Improve:

; - checking a down ladder at the bottom of the level (outside level definition)
; - player priority when in a ladder and priority of the coin animation
; - laser barriers should have a disabled state (instead of using the animation index)

; - entering water with stick down increase speed?
; - too slow, jump too low?
; - too much inertia? test without inertia
; - scale of the level too small
; - hide last vscrol scan line with black color + DLI?

; - half jump side speed after a landing
; - small control delay after landing from a big jump
; - collision/movement priority (side or vertical)
; - different max side speed when falling
; - side speed deaccel when falling

; + animations: vertical laser barrier, coin rotation, enemy 1 (cannon core)


; ToDo:

; - add collectable coin in water

; - recollect logic counter

; - shoot (positions, visual slots, collisions), special list

; - activate enemies, shooting, destruction, populate special list at the start of the level
;  (to manage behaviors, create shots when near player..)

; - activate laser barriers
; - danger zones logic (on / off timers), special list

; - try different type of camera, with push limits and platform snapping

; - different types of controls, keyboard

; - running state?
; - double jump
; - jump down from platform
; - wall jump
; - wall climbing/descend, slide (grab wall?)
; - glide
; - grab ceiling (like rope hanging), move only to the sides
; - jump down attack or while in air (camera shake?).. jump up attack? to the sides?
; - jetpack mode

; - moving platform
; - hidden tunnels, breakable walls/platforms (by jump attacks, shots)
; - vanishing platforms (by time over them)
; - morphing/moving blocks/platforms
; - on/off platforms/walls (like laser barriers)
; - jump pads
; - velocity changing platforms (infinite bands, mud..)
; - platforms with inclination or bumps

; - switches to change the state of things, platforms

; - special items, objects, inventory

; - player animations


;--------------------------------------------------------------------------------
; includes:

	icl "base/sys_equates.asm"

	icl "base/sys_macros.asm"


;--------------------------------------------------------------------------------
; variables:

	org Vars_address

m_dliSaveA					org *+1
m_dliSaveX					org *+1
m_dliSaveY					org *+1

m_redrawPlayerCounter		org *+1

m_playerBaseColor			org *+1

; left side coordinate
m_playerLevelLeftX_H2		org *+1
m_playerLevelLeftX_H1		org *+1
m_playerLevelLeftX_L1		org *+1

; right side coordinate
m_playerLevelRightX_H2		org *+1
m_playerLevelRightX_H1		org *+1
m_playerLevelRightX_L1		org *+1

; top line coordinate
m_playerLevelTopY_H2		org *+1
m_playerLevelTopY_H1		org *+1
m_playerLevelTopY_L1		org *+1

; bottom line coordinate
m_playerLevelBottomY_H2		org *+1
m_playerLevelBottomY_H1		org *+1
m_playerLevelBottomY_L1		org *+1

m_playerScreenLeftX			org *+1
m_playerScreenTopY			org *+1

m_oldPlayerScreenLeftX		org *+1
m_oldPlayerScreenTopY		org *+1

m_playerSizeX				org *+1
m_playerSizeY				org *+1
m_playerSizeXMinusOne		org *+1
m_playerSizeYMinusOne		org *+1


m_playerSideSpeed_H1		org *+1
m_playerSideSpeed_L1		org *+1

m_playerMaxSideSpeed_H1		org *+1
m_playerMaxSideSpeed_L1		org *+1

m_playerSideAccel_H1		org *+1
m_playerSideAccel_L1		org *+1

m_playerSideDeaccel_H1		org *+1
m_playerSideDeaccel_L1		org *+1


m_playerSpeedY_H1			org *+1
m_playerSpeedY_L1			org *+1

m_playerJumpSpeed_H1		org *+1
m_playerJumpSpeed_L1		org *+1

m_playerMaxFallSpeed_H1		org *+1
m_playerMaxFallSpeed_L1		org *+1

m_playerGravityAccel_H1		org *+1
m_playerGravityAccel_L1		org *+1

m_playerWaterAccelY_H1		org *+1
m_playerWaterAccelY_L1		org *+1
m_playerMaxWaterSpeedY_H1	org *+1
m_playerMaxWaterSpeedY_L1	org *+1
m_playerMoveWaterSpeedY_H1	org *+1
m_playerMoveWaterSpeedY_L1	org *+1

m_playerHoldJumpCounter		org *+1

; for now this is used to disable the jump for some frames, after entering the water
m_playerNoJumpTimer			org *+1

m_lastPlayerSideDirection	org *+1
; 0 - none, 1 - right, 2 - left
m_lastPlayerShootDirection	org *+1

m_playerOverPlatformFlag	org *+1
m_playerTopCollisionFlag	org *+1
m_playerSideCollisionFlag	org *+1

m_playerState				org *+1
; 0 - static, 1 - over platform, 2 - jumping up, 3 - falling,
; 4 - climbing, 5 - in water, 6 - gravity test

m_playerMethodPointer		org *+2

m_pageZeroTempPointer		org *+2
m_pageZeroTempValue			org *+1


m_ptrLevelLine				org *+2

m_levelCharSizeX			org *+1
m_levelCharSizeY			org *+1
; char size goes from 1 to 255 characters, 0 = 256 characters

m_levelSizeX_H2				org *+1
m_levelSizeX_H1				org *+1

m_levelSizeY_H2				org *+1
m_levelSizeY_H1				org *+1

m_levelWindowX_H2			org *+1
m_levelWindowX_H1			org *+1

m_levelWindowY_H2			org *+1
m_levelWindowY_H1			org *+1


m_stick0					org *+1
m_oldStick0					org *+1
m_currentTrigger0			org *+1
m_oldTrigger0				org *+1
m_triggerState				org *+1
; 0 - not pressed, 1 - just released, 2 - just pressed, 3 - pressed


;--------------------------------------------------------------------------------
; memory map

Vars_address			= 128

Code_address			= $5000		; 20K (22K zone)

Standard_Font_address	= $9C00		; 39K (1K size)

PM_address				= $A000		; 40K (2K size - 768 bytes)

Font1_address			= $A800		; 42K (1K size)

DL1_address				= $AC00		; 43K (<1K size)
DL1_data_address		= $B000		; 44K (4K size)


;--------------------------------------------------------------------------------
; constants

;PAL_VERSION				= 1
; remember, when going from NTSC to PAL:
; colors - 1 (same lum) .. here only color 7 (water blue) was left equal
; times multiplied by (5/6)
; speeds and accelerations multiplied by (6/5) .. maybe accelerations requires another adjustment
; some animations speed maybe should not (cannot) be converted, if they are not related to gameplay
; (for example the timing for the laser enabled/disabled states was converted)
; also, one extra precison byte (_L2) could be useful to get a better conversion in some cases

MAX_ROWS				= 40

PM_GRACTL				= 3
PRIOR_OPTIONS			= 1+16		; use 8+16 if we want the player behind the coin animations (but also behind the ladders)

CHAR_PIXELS_X			= 4
CHAR_PIXELS_Y			= 8

; max level size now should be 256 x 255 lines
LEVEL_CHAR_SIZE_X		= 82
LEVEL_CHAR_SIZE_Y		= 48

WINDOW_CHAR_SIZE_X		= 40
WINDOW_CHAR_SIZE_Y		= 24

PM_BASE_SCREEN_X		= 48
PM_BASE_SCREEN_Y		= 32

; player side direction
PSD_NONE				= 0
PSD_RIGHT				= 1
PSD_LEFT				= 2

; player states
PS_STATIC				= 0
PS_OVER_PLATFORM		= 1
PS_JUMPING				= 2
PS_FALLING				= 3
PS_CLIMBING				= 4
PS_IN_WATER				= 5
PS_GRAVITY_TEST			= 6
PS_LOSE					= 7

; stick directions
// STICK_MASK_RIGHT		= %1000
// STICK_MASK_LEFT		= %0100
// STICK_MASK_DOWN		= %0010
// STICK_MASK_UP		= %0001

; trigger states
TS_NOT_PRESSED			= 0
TS_JUST_RELEASED		= 0
TS_PRESSED				= 0
TS_JUST_PRESSED			= 0

; character types
CT_BLOCK				= 1
CT_COLLECTABLE			= 2
CT_DANGER				= 3
CT_LADDER				= 4
CT_WATER				= 5

ENEMY_SHOOT_DELTA_Y		= 32


.ifndef PAL_VERSION

TENTH_OF_SECOND_FRAMES	= 6

MINIMUM_CAMERA_STEP		= 1

JUMP_HOLD_FRAMES		= 8

JUMP_HSPEED_SLOWDOWN	= 64

WATER_NO_JUMP_FRAMES	= 30

; 001.160 --> 97.5 pixels per second (at 60 fps only!)
SIDE_SPEED_LAND_H1		= 1
SIDE_SPEED_LAND_L1		= 64
; 6 frames (at 60 fps only!) or 0.x seconds, to reach max speed
SIDE_ACCEL_LAND_H1		= 0
SIDE_ACCEL_LAND_L1		= 32
; 3 frames (at 60 fps only!) or 0.x seconds, to reach zero speed
SIDE_DEACCEL_LAND_H1	= 0
SIDE_DEACCEL_LAND_L1	= 128

;
SIDE_SPEED_WATER_H1		= 0
SIDE_SPEED_WATER_L1		= 128
; _ frames (at 60 fps only!) or _ seconds, to reach max speed
SIDE_ACCEL_WATER_H1		= 0
SIDE_ACCEL_WATER_L1		= 4
; _ frames (at 60 fps only!) or _ seconds, to reach zero speed
SIDE_DEACCEL_WATER_H1	= 0
SIDE_DEACCEL_WATER_L1	= 4

; 001.120 --> 88.125 pixels per second (at 60 fps only!)
SIDE_SPEED_AIR_H1		= 1
SIDE_SPEED_AIR_L1		= 32
; 17.3 frames (at 60 fps only!) or 0.288 seconds, to reach max speed
SIDE_ACCEL_AIR_H1		= 0
SIDE_ACCEL_AIR_L1		= 16
; 17.3 frames (at 60 fps only!) or 0.288 seconds, to reach zero speed
SIDE_DEACCEL_AIR_H1		= 0
SIDE_DEACCEL_AIR_L1		= 16

;
CLIMB_UP_SPEED_Y_H1		= 0
CLIMB_UP_SPEED_Y_L1		= 224
;
CLIMB_DOWN_SPEED_Y_H1	= 1
CLIMB_DOWN_SPEED_Y_L1	= 128

PLAYER_MISSILE_SPEED_X	= 2*256+128
ENEMY_MISSILE_SPEED_X	= 2*256
ENEMY_INIT_TIMER		= 120

VLASER_DISABLED_INDEX	= 60

.else

TENTH_OF_SECOND_FRAMES	= 5

MINIMUM_CAMERA_STEP		= 1

JUMP_HOLD_FRAMES		= 6		; 6.67 for pal

JUMP_HSPEED_SLOWDOWN	= 77

WATER_NO_JUMP_FRAMES	= 25

;  pixels per second (at 50 fps only!)
SIDE_SPEED_LAND_H1		= 1
SIDE_SPEED_LAND_L1		= 128
;  frames (at 50 fps only!) or 0.x seconds, to reach max speed
SIDE_ACCEL_LAND_H1		= 0
SIDE_ACCEL_LAND_L1		= 38
;  frames (at 50 fps only!) or 0.x seconds, to reach zero speed
SIDE_DEACCEL_LAND_H1	= 0
SIDE_DEACCEL_LAND_L1	= 154

;
SIDE_SPEED_WATER_H1		= 0
SIDE_SPEED_WATER_L1		= 154
; _ frames (at 50 fps only!) or _ seconds, to reach max speed
SIDE_ACCEL_WATER_H1		= 0
SIDE_ACCEL_WATER_L1		= 5
; _ frames (at 50 fps only!) or _ seconds, to reach zero speed
SIDE_DEACCEL_WATER_H1	= 0
SIDE_DEACCEL_WATER_L1	= 5

;  pixels per second (at 50 fps only!)
SIDE_SPEED_AIR_H1		= 1
SIDE_SPEED_AIR_L1		= 90
;  frames (at 50 fps only!) or  seconds, to reach max speed
SIDE_ACCEL_AIR_H1		= 0
SIDE_ACCEL_AIR_L1		= 19
;  frames (at 50 fps only!) or  seconds, to reach zero speed
SIDE_DEACCEL_AIR_H1		= 0
SIDE_DEACCEL_AIR_L1		= 19

;
CLIMB_UP_SPEED_Y_H1		= 1
CLIMB_UP_SPEED_Y_L1		= 13
;
CLIMB_DOWN_SPEED_Y_H1	= 1
CLIMB_DOWN_SPEED_Y_L1	= 205

PLAYER_MISSILE_SPEED_X	= 3*256
ENEMY_MISSILE_SPEED_X	= 2*256+104
ENEMY_INIT_TIMER		= 100

VLASER_DISABLED_INDEX	= 50

.endif

;--------------------------------------------------------------------------------

	org Code_address

InitSystem
	ClearSystem

	DisableBasic
	DisableOperativeSystem

; only run in the correct system
	lda PAL
	and #14

.if .def PAL_VERSION
NO_PAL_loop
	bne NO_PAL_loop
.else
NO_NTSC_loop
	beq NO_NTSC_loop
.endif
	
	jsr Init_NMI_Handler_DLI1_Hack

	;SetMemory DL1_data_address1, 1024, 0


	VcountWait 120


	jsr InitVars

	jsr InitMissileSystem


InitHardware

.ifndef PAL_VERSION
	SetColor 0, 0, 12		; white (platforms, scenary)
	SetColor 1, 1, 14		; yellow (collectables) .. this is changed in the DLI also!
	SetColor 2, 7, 2		; blue (water) .. this is changed in the DLI also!
	SetColor 3, 2, 6		; red (danger)
	SetColor 4, 0, 0		; black (background)

	SetPMColor 0, 11, 10	; player
	SetPMColor 1, 2, 10		; enemy explosion
.else
	SetColor 0, 0, 12		; white (platforms, scenary)
	SetColor 1, 15, 14		; yellow (collectables) .. this is changed in the DLI also!
	SetColor 2, 7, 2		; blue (water) .. this is changed in the DLI also!
	SetColor 3, 1, 6		; red (danger)
	SetColor 4, 0, 0		; black (background)

	SetPMColor 0, 10, 10	; player
	SetPMColor 1, 1, 10		; enemy explosion
.endif

; set the player info
	lda #0
	sta SIZEP0

	lda #%01010101		; double width for all missiles
	sta SIZEM

	SetPMBaseAddress PM_address

	lda #PM_GRACTL
	sta GRACTL

	lda #PRIOR_OPTIONS
	sta PRIOR

	lda #0
	sta HITCLR

	lda #12
	sta HSCROL
	lda #0
	sta VSCROL

	SetFontAddress Font1_address

	SetDisplayListAddress DL1_address

	;SetVector VDSLST, DLI_line0

	lda #128		; enable DLI's (but no VBI's)
	sta NMIEN

	lda #[DV_DMA_ON | DV_NORMAL_PF | DV_PLAYERS_ON | DV_MISSILES_ON | DV_PM_ONE_LINE]
	sta DMACTL


	VcountWait 120


;================================================================================

DemoLoop

	;SetColor 4, 0, 14
	jsr CheckInput
	
	jsr UpdateTimers

	jmp (m_playerMethodPointer)
	
PlayerMethodReturn
	;SetColor 4, 3, 4


	lda m_playerState
	cmp #PS_LOSE
	beq PlayerEndStates


PlayerNormalStates
	jsr UpdateCameraWindow
	;SetColor 4, 6, 14

	jsr SetPlayerScreenPos

	jsr DrawPlayer
	;SetColor 4, 9, 4


EnemyUpdate
	jsr UpdateEnemyManager


MissilesStep
	jsr UpdateMissileSystem

	
	jsr DrawEnemyExplosion

	
GameAnimations
	jsr DoFontAnimations

	jsr UpdateCoinAnimations
	;SetColor 4, 0, 0

	jsr UpdateInfoLine


	VcountWait 120


	jsr CheckPMCollisions


	jmp DemoLoop


;----------------------------------------
PlayerEndStates
	jsr DrawPlayerExplosion

	jsr DoFontAnimations

	jsr UpdateCoinAnimations

	jsr UpdateMissileSystem
	
	jsr DrawEnemyExplosion
	;SetColor 4, 0, 0

	jsr UpdateInfoLine


	VcountWait 120


	lda #0
	sta HITCLR


	jmp DemoLoop


;================================================================================

; display 2 digits with values from 00 to FF
; passs the value in A and the line row in Y

DisplayDebugInfoHexFF
	stx m_debugSaveX

	sta DDIH_save_value+1

; display 2 digits (from 0 to F)
	lsr
	lsr
	lsr
	lsr
	tax
	lda TabHexNibbleToScreenDigit,x
	sta DL1_info_line_address,y

DDIH_save_value
	lda #$FF

	and #15
	tax
	lda TabHexNibbleToScreenDigit,x
	sta DL1_info_line_address+1,y

	ldx m_debugSaveX

	rts


; display 2 digits with values from 00 to 99
; passs the value in A and the line row in Y

DisplayDebugInfoBinary99
	stx m_debugSaveX

	tax
	cpx #100
	bcc NoOverflow
	ldx #99

NoOverflow
	lda TabBinaryToBCD,x
	tax

; display 2 digits (from 0 to 9)
	lsr
	lsr
	lsr
	lsr
	ora #16			; add the "0" character value
	sta DL1_info_line_address,y

	txa
	and #15
	ora #16			; add the "0" character value
	sta DL1_info_line_address+1,y

	ldx m_debugSaveX

	rts


DisplayDebugInfoBinary9
	cmp #10
	bcc NoOverflow2
	lda #9

NoOverflow2
; display 1 digit (from 0 to 9)
	ora #16			; add the "0" character value
	sta DL1_info_line_address,y

	rts


ClearDebugLineInfo
	stx m_debugSaveX

	lda #0
	tax

CDI_loop
	sta DL1_info_line_address,x
	inx
	cpx #40
	bne CDI_loop

	ldx m_debugSaveX

	rts


m_debugSaveX		.byte 0


;--------------------------------------------------------------------------------

TabHexNibbleToScreenDigit
	.sb "0123456789ABCDEF"

TabBinaryToBCD
	.byte $00, $01, $02, $03, $04, $05, $06, $07, $08, $09
	.byte $10, $11, $12, $13, $14, $15, $16, $17, $18, $19
	.byte $20, $21, $22, $23, $24, $25, $26, $27, $28, $29
	.byte $30, $31, $32, $33, $34, $35, $36, $37, $38, $39
	.byte $40, $41, $42, $43, $44, $45, $46, $47, $48, $49
	.byte $50, $51, $52, $53, $54, $55, $56, $57, $58, $59
	.byte $60, $61, $62, $63, $64, $65, $66, $67, $68, $69
	.byte $70, $71, $72, $73, $74, $75, $76, $77, $78, $79
	.byte $80, $81, $82, $83, $84, $85, $86, $87, $88, $89
	.byte $90, $91, $92, $93, $94, $95, $96, $97, $98, $99


;================================================================================

InitVars

; input info
	lda #1
	sta m_currentTrigger0
	sta m_oldTrigger0

	lda #0
	sta m_triggerState

	lda #15
	sta m_stick0
	sta m_oldStick0


; level info
	lda #>[CHAR_PIXELS_X]
	sta m_levelWindowX_H2
	lda #<[CHAR_PIXELS_X]
	sta m_levelWindowX_H1
	lda #0
	sta m_levelWindowY_H2
	lda #0
	sta m_levelWindowY_H1

	lda #LEVEL_CHAR_SIZE_X
	sta m_levelCharSizeX
	lda #LEVEL_CHAR_SIZE_Y
	sta m_levelCharSizeY

	lda #>[LEVEL_CHAR_SIZE_X*CHAR_PIXELS_X]
	sta m_levelSizeX_H2
	lda #<[LEVEL_CHAR_SIZE_X*CHAR_PIXELS_X]
	sta m_levelSizeX_H1

	lda #>[LEVEL_CHAR_SIZE_Y*CHAR_PIXELS_Y]
	sta m_levelSizeY_H2
	lda #<[LEVEL_CHAR_SIZE_Y*CHAR_PIXELS_Y]
	sta m_levelSizeY_H1

	lda #<DL1_data_address
	sta m_currentLineAddress
	lda #>DL1_data_address
	sta m_currentLineAddress+1
	jsr InitLevelTable


	jsr InitEnemyManager

	jsr InitGoldCounter
	

; player info

; remember that we don't use any speed over 1 char per frame right now,
; because we don't want to lose collisions (in that case case we should
; use 2 or more steps per frame, or a "sweep" collision method)

.ifndef PAL_VERSION
; air vertical movement
	lda #3		; 240 pixels per second (at 60 fps only!)
	sta m_playerJumpSpeed_H1
	lda #128
	sta m_playerJumpSpeed_L1

	lda #6		; near 400 pixels per second (1 character per frame at 50 fps, PAL friendly)
	sta m_playerMaxFallSpeed_H1
	lda #170
	sta m_playerMaxFallSpeed_L1

	lda #0		; 12.8 frames (at 60 fps only!) to reach max speed (from zero)
	sta m_playerGravityAccel_H1
	lda #112
	sta m_playerGravityAccel_L1

; water vertical movement
	lda #0
	sta m_playerWaterAccelY_H1
	lda #32
	sta m_playerWaterAccelY_L1

	lda #0
	sta m_playerMaxWaterSpeedY_H1
	lda #128
	sta m_playerMaxWaterSpeedY_L1

	lda #1
	sta m_playerMoveWaterSpeedY_H1
	lda #0
	sta m_playerMoveWaterSpeedY_L1
.else
; air vertical movement
	lda #4		;  pixels per second (at 50 fps only!)
	sta m_playerJumpSpeed_H1
	lda #51
	sta m_playerJumpSpeed_L1

	lda #7		; near 400 pixels per second (1 character per frame at 50 fps, PAL friendly)
	sta m_playerMaxFallSpeed_H1
	lda #255
	sta m_playerMaxFallSpeed_L1

	lda #0		;  frames (at 50 fps only!) to reach max speed (from zero)
	sta m_playerGravityAccel_H1
	lda #134
	sta m_playerGravityAccel_L1

; water vertical movement
	lda #0
	sta m_playerWaterAccelY_H1
	lda #38
	sta m_playerWaterAccelY_L1

	lda #0
	sta m_playerMaxWaterSpeedY_H1
	lda #154
	sta m_playerMaxWaterSpeedY_L1

	lda #1
	sta m_playerMoveWaterSpeedY_H1
	lda #51
	sta m_playerMoveWaterSpeedY_L1
.endif

; set size in pixels
	ldx #4
	stx m_playerSizeX
	dex
	stx m_playerSizeXMinusOne
	ldx #8
	stx m_playerSizeY
	dex
	stx m_playerSizeYMinusOne


	jsr InitPlayerInfo


	rts


;================================================================================

InitPlayerInfo

	lda #0
	sta m_playerOverPlatformFlag
	sta m_playerTopCollisionFlag
	sta m_playerSideCollisionFlag
	sta m_playerHoldJumpCounter
	sta m_playerNoJumpTimer

	ldx #PS_OVER_PLATFORM
	jsr SetPlayerMethod

	lda #0
	sta m_redrawPlayerCounter


	lda #0
	sta m_playerSideSpeed_H1
	sta m_playerSideSpeed_L1


	lda #PSD_NONE
	sta m_lastPlayerSideDirection
	lda #PSD_RIGHT
	sta m_lastPlayerShootDirection


; set spawn position
	lda #0
	sta m_playerLevelLeftX_H2
	lda #16
	sta m_playerLevelLeftX_H1
	lda #0
	sta m_playerLevelLeftX_L1	; fractional part

	lda #0
	sta m_playerLevelBottomY_H2
	lda #[9*8+7]
	sta m_playerLevelBottomY_H1
	lda #0
	sta m_playerLevelBottomY_L1	; fractional part

	lda #0
	sta m_playerLevelRightX_L1	; fractional part (not used)
	sta m_playerLevelTopY_L1		; fractional part (not used)

	jsr SetPlayerInfo


	rts


;================================================================================

InitLevelTable
	ldy #0

ILTL_loop
	lda m_currentLineAddress
	sta TabLevelLineAddressLSB,y
	lda m_currentLineAddress+1
	sta TabLevelLineAddressMSB,y

	lda m_levelCharSizeX
	bne ILTL_below_256

	inc m_currentLineAddress+1	; if m_levelCharSizeX = 0, then add 256 chars
	jmp ILTL_check_loop

ILTL_below_256
	clc
	adc m_currentLineAddress		; from 1 to 255 chars
	sta m_currentLineAddress

	lda m_currentLineAddress+1
	adc #0
	sta m_currentLineAddress+1

ILTL_check_loop
	iny
	cpy m_levelCharSizeY		; from 1 to 255 lines, 0 = 256 lines in this case
	bne ILTL_loop

	; add one more line definition to fix problem of ladders in the last line, but this is not going to work if there are 256 lines!
	lda m_currentLineAddress
	sta TabLevelLineAddressLSB,y
	lda m_currentLineAddress+1
	sta TabLevelLineAddressMSB,y
	
	rts


m_currentLineAddress		.word 0


;================================================================================

InitGoldCounter
	ldx #0		; line index

IGC_loop1
	ldy #0		; column index
	
	lda TabLevelLineAddressLSB,x
	sta m_ptrLevelLine
	lda TabLevelLineAddressMSB,x
	sta m_ptrLevelLine+1

IGC_loop2
	lda (m_ptrLevelLine),y
	
	stx m_pageZeroTempValue
	tax
	lda TabCharDefinition,x
	ldx m_pageZeroTempValue
	
	and #15
	cmp #CT_COLLECTABLE
	bne IGC_no_gold
	
	inc m_goldCounter

IGC_no_gold
	iny
	cpy m_levelCharSizeX
	bne IGC_loop2

IGC_check_loop_Y
	inx
	cpx m_levelCharSizeY
	bne IGC_loop1
	
	rts


m_goldCounter .byte 0		; assuming less than 256


;================================================================================

UpdateInfoLine
	lda m_goldCounter		; assuming 99 or less
	ldy #4
	jsr DisplayDebugInfoBinary99

	lda m_activeEnemies
	ldy #12
	jsr DisplayDebugInfoBinary99

; update the time here also
	lda m_disableGameTimer
	bne UIL_exit
	
	lda m_goldCounter
	bne UIL_update_timer
	lda m_activeEnemies
	bne UIL_update_timer
	
	lda #1
	sta m_disableGameTimer		; my record? 1:16.1 (NTSC, lost one life)

; allow the final update..

UIL_update_timer
	inc m_gameTSFrameCounter
	lda m_gameTSFrameCounter
	cmp #TENTH_OF_SECOND_FRAMES
	bne UIL_draw_timer
	
	lda #0
	sta m_gameTSFrameCounter
	inc m_gameTimerTSeconds
	lda m_gameTimerTSeconds
	cmp #10
	bne UIL_draw_timer
	
	lda #0
	sta m_gameTimerTSeconds
	inc m_gameTimerSeconds
	lda m_gameTimerSeconds
	cmp #60
	bne UIL_draw_timer
	
	lda #0
	sta m_gameTimerSeconds
	inc m_gameTimerMinutes

	lda m_gameTimerMinutes
	cmp #100
	bne UIL_draw_timer
	
	lda #1
	sta m_disableGameTimer
	jmp UIL_exit				; final timer would be 99:99.9 in this case

UIL_draw_timer
	lda m_gameTimerMinutes
	ldy #20
	jsr DisplayDebugInfoBinary99
	
	lda m_gameTimerSeconds
	ldy #23
	jsr DisplayDebugInfoBinary99
	
	lda m_gameTimerTSeconds
	ldy #26
	jsr DisplayDebugInfoBinary9

UIL_exit
	rts


m_gameTSFrameCounter .byte 0
m_gameTimerMinutes .byte 0
m_gameTimerSeconds .byte 0
m_gameTimerTSeconds .byte 0
m_disableGameTimer .byte 0


;================================================================================

	icl "player_states.asm"


;================================================================================

	icl "player_movement.asm"


;================================================================================
;================================================================================

UpdateTimers
	lda m_playerNoJumpTimer
	beq UT_end_no_jump

	dec m_playerNoJumpTimer
	
UT_end_no_jump
	rts
		

;================================================================================
;================================================================================

CheckInput

CI_check_stick
	lda m_stick0
	sta m_oldStick0

	lda PORTA
	and #15
	sta m_stick0

CI_check_shoot_right
	cmp #%1000	; right
	bcs CI_check_shoot_left
	lda #PSD_RIGHT
	sta m_lastPlayerShootDirection
	jmp CI_check_trigger

CI_check_shoot_left
	and #%0100	; left
	bne CI_check_trigger
	lda #PSD_LEFT
	sta m_lastPlayerShootDirection

CI_check_trigger
	lda m_currentTrigger0
	sta m_oldTrigger0

	lda TRIG0
	sta m_currentTrigger0

; 0 - not pressed, 1 - just released, 2 - just pressed, 3 - pressed
	asl
	ora m_oldTrigger0
	eor #%11
	sta m_triggerState

; reset trigger latches
	lda #0+PM_GRACTL
	sta GRACTL			; reset triggers latch mode
	lda #4+PM_GRACTL
	sta GRACTL			; set triggers latch mode

	rts


;================================================================================
;================================================================================

CheckPMCollisions
	lda #0
	sta TabMissileHWCollision
	sta TabMissileHWCollision+1
	sta TabMissileHWCollision+2
	sta TabMissileHWCollision+3


CPMC_check_p0_pf3
	lda P0PF
	and #%1000
	beq CPMC_check_m2_pl0

	ldx #PS_LOSE
	jsr SetPlayerMethod
	jmp CPMC_check_m0_pf0_3

CPMC_check_m2_pl0
	lda M2PL
	and #%0001
	beq CPMC_check_m3_pl0

	lda #3
	sta TabMissileHWCollision+2

	ldx #PS_LOSE
	jsr SetPlayerMethod
	jmp CPMC_check_m0_pf0_3

CPMC_check_m3_pl0
	lda M3PL
	and #%0001
	beq CPMC_check_m0_pf0_3

	lda #3
	sta TabMissileHWCollision+3

	ldx #PS_LOSE
	jsr SetPlayerMethod
	

CPMC_check_m0_pf0_3
	lda M0PF
	and #%1001
	beq CPMC_check_m1_pf0_3

	lda #1
	sta TabMissileHWCollision

CPMC_check_m1_pf0_3
	lda M1PF
	and #%1001
	beq ClearPMCollisions

	lda #1
	sta TabMissileHWCollision+1

; CPMC_check_m2_pf0_3
; 	lda M2PF
; 	and #%1001
; 	beq CPMC_check_m3_pf0_3
;
; 	lda #1
; 	sta TabMissileHWCollision+2
;
; CPMC_check_m3_pf0_3
; 	lda M3PF
; 	and #%1001
; 	beq CPMC_check_m2_pl0
;
; 	lda #1
; 	sta TabMissileHWCollision+3

/*
CPMC_check_m2_pl0
	lda M2PL
	and #%0001
	beq CPMC_check_m3_pl0

	lda #3
	sta TabMissileHWCollision+2

CPMC_check_m3_pl0
	lda M3PL
	and #%0001
	beq ClearPMCollisions

	lda #3
	sta TabMissileHWCollision+3
*/

ClearPMCollisions
	lda #0
	sta HITCLR

	rts


;================================================================================

	icl "missile_system.asm"
	
;================================================================================

	icl "animations_logic.asm"
	
;================================================================================

	icl "camera_logic.asm"

;================================================================================

	icl "enemy_manager.asm"

;================================================================================
; My first NMI handler.. (O.S. classic handler)
;================================================================================

; NMI_Handler
; 	bit NMIST
; 	bpl VBI
;
; DLI	jmp (VDSLST)
;
; VBI	cld
; 	pha
; 	txa
; 	pha
; 	tya
; 	pha
; 	sta NMIRES
; 	jmp (VVBLKI)

;--------------------------------------------------------------------------------
; call this only after disabling the operative system rom area !!

; Init_NMI_Handler
; 	SetVector NMIH_VECTOR, NMI_Handler
;
; 	rts

Init_NMI_Handler_DLI1_Hack
	SetVector NMIH_VECTOR, DLI_1

	rts


;================================================================================

;----------------------------------------
; display list interruptions code
;----------------------------------------

DLI_1
	sta m_dliSaveA

; 	lda #0
; DLI_loop
; 	sta WSYNC
; 	sta COLBK
;
; 	sta WSYNC
; 	sta WSYNC
; 	sta WSYNC
; 	sta WSYNC
; 	sta WSYNC
; 	sta WSYNC
; 	sta WSYNC
;
; 	clc
; 	adc #16
; 	bne DLI_loop
;
; 	sta WSYNC
; 	sta COLBK


; 	lda m_playerBaseColor
; 	sta COLPM0
;
; 	lda m_redrawPlayerCounter
; 	beq DLI1_exit
;
; 	;stx m_dliSaveX
; 	sty m_dliSaveY
;
; 	lda m_playerScreenLeftX
; 	sta HPOSP0
; 
; 	;ldx m_playerSizeY
; 
; 	lda #0
; 	sta m_redrawPlayerCounter
; 
; 	ldy m_oldPlayerScreenTopY
; 	sta p0_area_1,y
; 	sta p0_area_1+1,y
; 	sta p0_area_1+2,y
; 	sta p0_area_1+3,y
; 	sta p0_area_1+4,y
; 	sta p0_area_1+5,y
; 	sta p0_area_1+6,y
; 	sta p0_area_1+7,y
; 
; 	ldy m_playerScreenTopY
; 	lda TabPlayerShape_01
; 	sta p0_area_1,y
; 	lda TabPlayerShape_01+1
; 	sta p0_area_1+1,y
; 	lda TabPlayerShape_01+2
; 	sta p0_area_1+2,y
; 	lda TabPlayerShape_01+3
; 	sta p0_area_1+3,y
; 	lda TabPlayerShape_01+4
; 	sta p0_area_1+4,y
; 	lda TabPlayerShape_01+5
; 	sta p0_area_1+5,y
; 	lda TabPlayerShape_01+6
; 	sta p0_area_1+6,y
; 	lda TabPlayerShape_01+7
; 	sta p0_area_1+7,y
;
; 	;ldx m_dliSaveX
; 	ldy m_dliSaveY

	SetColor 1, 0, 12
	SetColor 2, 0, 2

	sta WSYNC
	SetFontAddress Standard_Font_address
	:8 sta WSYNC		; optimize this using another dli

	SetFontAddress Font1_address

.ifndef PAL_VERSION
	SetColor 1, 1, 14		; yellow (collectables)
	SetColor 2, 7, 2		; blue (water)
.else
	SetColor 1, 15, 14		; yellow (collectables)
	SetColor 2, 7, 2		; blue (water)
.endif

DLI1_exit
	lda m_dliSaveA

	rti


;================================================================================

TabPlayerShapeAddressLSB
	.byte <TabPlayerShape_01, <TabPlayerShape_02

TabPlayerShapeAddressMSB
	.byte >TabPlayerShape_01, >TabPlayerShape_02

TabPlayerShapeOffsetX
	.byte 0, 255		; 255 means -1 here

TabPlayerShapeSizeY
	.byte 8, 5

TabPlayerShape_01
	.byte %11110000
	.byte %11110000
	.byte %11110000
	.byte %11110000
	.byte %11110000
	.byte %11110000
	.byte %11110000
	.byte %11110000

TabPlayerShape_02
	.byte %11111100
	.byte %11111100
	.byte %11111100
	.byte %11111100
	.byte %11111100


;================================================================================

END_CODE_WARNING
	.if END_CODE_WARNING > Standard_Font_address
		.error "Code overrides fixed data area!"
	.endif


;================================================================================
;================================================================================

	org Standard_Font_address

	ins "data/atari.fnt"


;================================================================================

	org PM_address


TabLevelLineAddressLSB
	:256 .byte 0

TabLevelLineAddressMSB
	:256 .byte 0


;----------------------------------------
; Type definition: low 4 bits (0-15 value)
; - 0: empty normal space (with gravity info?)
; - 1: block (with collision info, RLDU)
; - 2: collectable (with value info?)
; - 3: danger (with energy info?)
; - 4: ladder, climbable (with directions info?)
; - 5: water (with density info? currents?)

; - x: special items
; - x: wind, force fields
; - x: jump pads
; - x: ice, speed blocks


;----------------------------------------
TabCharDefinition
	.byte 0

; full
	.byte %11110000+1
	.byte %11110000+1
	.byte %00000000+5		; water

; dither 1
	.byte %11110000+1
	.byte %11110000+1
	.byte %11110000+1

; dither 2
	.byte %11110000+1
	.byte %11110000+1
	.byte %11110000+1

; collectable 1
	.byte %00000000+2

; ladder
	.byte %00000000+4

; water top
	.byte %00010000+5
	.byte %00010000+5

; vertical line dither
	.byte %11110000+1
	.byte %11110000+1
	.byte %11110000+1

; blue spikes
	.byte %00000000+3

; border 1
	.byte %11110000+1
	.byte %11110000+1

; border 2
	.byte %11110000+1
	.byte %11110000+1

; only top platform
	.byte %00010000+1

; horizontal dither 1
	.byte %00010000+1
; horizontal dither 2
	.byte %00010000+1
; horizontal dither 3 (barrier)
	.byte %00010000+1

	:6 .byte 0
	:32 .byte 0

; enemy 1 left side (blue)
	.byte %00010000+3
; enemy 1 right side (blue)
	.byte %00010000+3

	:30 .byte 0
	:32 .byte 0


;----------------------------------------
; second half, inverse characters, fifth color (red, danger)
	org TabCharDefinition+128

	.byte 0

; full
	.byte %11110000+1
	.byte %11110000+1
	.byte %00000000+3		; lava

; dither 1
	.byte %11110000+1
	.byte %11110000+1
	.byte %11110000+1

; dither 2
	.byte %11110000+1
	.byte %11110000+1
	.byte %11110000+1

; collectable 1
	.byte %00000000+2

; ladder
	.byte %00000000+4

; lava top
	.byte %00000000+3
	.byte %00000000+3

; vertical line dither
	.byte %11110000+1
	.byte %11110000+1
	.byte %00000000+3		; animated vertical laser

; spikes
	.byte %00000000+3

; border 1
	.byte %11110000+1
	.byte %11110000+1

; border 2
	.byte %11110000+1
	.byte %11110000+1

; only top platform
	.byte %00010000+1

; horizontal dither 1
	.byte %00010000+1
; horizontal dither 2
	.byte %00010000+1
; horizontal dither 3 (laser barrier)
	.byte %00010000+3

	:6 .byte 0
	:32 .byte 0

; enemy 1 left side (red)
	.byte %00010000+3
; enemy 1 right side (red)
	.byte %00010000+3

	:30 .byte 0
	:32 .byte 0


;----------------------------------------
ms_area_1 = PM_address + 768

p0_area_1 = PM_address + 1024
p1_area_1 = p0_area_1 + 256
p2_area_1 = p1_area_1 + 256
p3_area_1 = p2_area_1 + 256


	org ms_area_1

	:1280 .byte 0


;================================================================================

	org Font1_address

	ins "data/pro1.fnt"

/*
; background (black): 0
	.byte 0,0,0,0,0,0,0,0

; full block (white, yellow, blue/red): 1,2,3
	:8 .byte %01010101

	:8 .byte %10101010

	:8 .byte %11111111

; full block, dither 1 (white, yellow, blue/red): 4,5,6
	:4 .byte %01000100, %00010001

	:4 .byte %10001000, %00100010

	:4 .byte %11001100, %00110011

; full block, dither 2 (white, yellow, blue/red): 7,8,9
	:2 .byte %01000000, %00000100, %00010000, %00000001

	:2 .byte %10000000, %00001000, %00100000, %00000010

	:2 .byte %11000000, %00001100, %00110000, %00000011

; collectable cross (yellow): 10
	.byte 0
	.byte %00100000
	.byte %00100000
	.byte %10001000
	.byte %10001000
	.byte %00100000
	.byte %00100000
	.byte 0

; ladder (yellow/blue): 11
	.byte %10000010
	.byte %10101010
	.byte %10000010
	.byte %10000010
	.byte %10000010
	.byte %10101010
	.byte %10000010
	.byte %10000010

; sea top (blue/red): 12,13
	.byte %00001111
	.byte %00111111
	.byte %11111111
	.byte %11111111
	.byte %11111111
	.byte %11111111
	.byte %11111111
	.byte %11111111

	.byte %00111100
	.byte %11000011
	.byte %11110000
	.byte %11111111
	.byte %11111111
	.byte %11111111
	.byte %11111111
	.byte %11111111

; vertical line, dither (white, yellow, blue/red): 14,15,16
	:4 .byte %00010000, %00000100

	:4 .byte %00100000, %00001000

	:4 .byte %00110000, %00001100

; spikes red: 17
	.byte %00110000
	.byte %00110000
	.byte %00110000
	.byte %00110000
	.byte %00110000
	.byte %11111100
	.byte %11111100
	.byte %11001100

; side border 1, top circle (white): 18,19
	.byte %00010101
	:7 .byte %01010101

	.byte %01010100
	:7 .byte %01010101

; side border 2, side circle (white): 20,21
	.byte %00010101
	:6 .byte %01010101
	.byte %00010101

	.byte %01010100
	:6 .byte %01010101
	.byte %01010100

; "only top" platform (white): 22
	.byte %01010101
	.byte %01010101
	.byte 0
	.byte %01000100
	.byte %00010001
	.byte %01000100
	.byte %00010001
	.byte %01000100


	.rept 105
		.byte 1,2,4,8,16,32,64,128
	.endr
*/

;================================================================================

; display list 1 data

	org DL1_address

	.byte DL_BLANK_8
	.byte DL_BLANK_8
	.byte DL_BLANK_8

DL1_LMS1
	?DL_data = DL1_data_address
	.rept 24
		.byte GM_CHAR_A4+DL_LMS_MASK+DL_HSCROLL_MASK+DL_VSCROLL_MASK
		.byte <?DL_data, >?DL_data
		?DL_data += 82
	.endr

	.byte GM_CHAR_A4+DL_LMS_MASK+DL_HSCROLL_MASK
	.byte <?DL_data, >?DL_data

	;.byte DL_BLANK_1+DL_DLI_MASK

	.byte DL_BLANK_2+DL_DLI_MASK

	?DL_data = DL1_info_line_address
	.byte GM_CHAR_G0+DL_LMS_MASK
	.byte <?DL_data, >?DL_data

; 	.rept 128
; 		.byte GM_PIXEL_G9+DL_LMS_MASK
; 		.byte <DL1_data_address, >DL1_data_address
; 	.endr

	.byte DL_JVB, <DL1_address, >DL1_address


DL1_info_line_address
.ifndef PAL_VERSION
	.sb "  G 00    E 00    T 00:00.0       NTSC  "
.else
	.sb "  G 00    E 00    T 00:00.0        PAL  "
.endif


;================================================================================

	org DL1_data_address

	ins "data/pro1.raw"

; add extra line info to avoid problem with ladder in the last line
	:LEVEL_CHAR_SIZE_X .byte 1
	
/*
	.byte 0

	.byte 1,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,1
	.byte 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0
	.byte 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0
	.byte 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0
	.byte 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0

	.byte 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0
	.byte 0,22,22,22,22, 22,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0
	.byte 0,0,0,11,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0
	.byte 0,0,0,11,0, 0,0,0,0,0, 22,22,22,22,22, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0
	.byte 0,0,0,11,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0

	.byte 0,0,0,11,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,20, 1,1,1,1,1, 1,1,1,1,1
	.byte 0,0,0,11,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,22, 22,22,22,22,22, 0,0,0,0,0, 144,0,0,0,0, 0,0,0,0,0
	.byte 0,0,0,11,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 144,0,0,0,10, 10,10,10,0,0
	.byte 0,0,0,11,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 144,0,0,0,10, 10,10,10,0,0
	.byte 0,0,0,11,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 144,0,0,0,10, 10,10,10,0,0

	.byte 0,0,0,11,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 144,0,0,0,10, 10,10,10,0,0
	.byte 0,0,0,11,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 144,0,0,0,0, 0,0,0,0,0
	.byte 0,0,0,11,0, 0,0,0,0,18, 19,19,19,0,0, 0,0,0,0,18, 1,1,1,1,1, 1,1,1,1,1, 1,1,1,19,145, 145,145,145,18,1
	.byte 0,0,0,11,0, 0,18,19,1,1, 4,4,1,0,0, 0,0,0,0,1, 4,4,4,4,4, 4,4,4,4,4, 4,4,4,4,4, 4,4,4,4,4
	.byte 1,18,19,1,19, 1,1,4,4,4, 4,4,1,12,13, 12,13,12,13,1, 4,4,4,4,4, 4,4,4,4,4, 4,4,4,4,4, 4,4,4,4,4

	.byte 4,4,4,4,4, 4,4,4,4,4, 4,4,1,3,3, 3,3,3,3,1, 4,4,7,7,7, 7,7,7,7,7, 7,7,7,7,7, 7,7,7,7,7
	.byte 7,7,7,7,7, 7,7,7,7,7, 4,4,1,3,3, 3,3,3,3,1, 4,4,7,7,7, 7,7,7,7,7, 7,7,7,7,7, 7,7,7,7,7
	.byte 7,7,7,7,7, 7,7,7,7,7, 4,4,1,6,6, 6,6,6,6,1, 4,4,7,7,7, 7,7,7,7,7, 7,7,7,7,7, 7,7,7,7,7
	.byte 7,7,7,7,7, 7,7,7,7,7, 4,4,1,9,9, 9,9,9,9,1, 4,4,7,7,7, 7,7,7,7,7, 7,7,7,7,7, 7,7,7,7,7
*/

; 	.byte %00000000,%00010001,%00100010,%00110011
; 	.byte %01000100,%01010101,%01100110,%01110111
; 	.byte %10001000,%10011001,%10101010,%10111011
; 	.byte %11001100,%11011101,%11101110,%11111111
; 	.byte 0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0


; external level definitions:
; - blocks ON/OFF
; - moving blocks
; - switches, triggers and traps
; - enemies and generators
; - dangers ON/OFF
; - moving dangers
; - interactions zones
; - text messages


;================================================================================

; run address

; 	org $2e0
; 	.word InitSystem

	run InitSystem



